/* mehPL:
 *    This is Open Source, but NOT GPL. I call it mehPL.
 *    I'm not too fond of long licenses at the top of the file.
 *    Please see the bottom.
 *    Enjoy!
 */

#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/io.h>
#include <stdint.h>
#include _BITHANDLING_HEADER_

#include "pinout.h"



void init_io(void);
//Adjust the volume, ish, of the attempted audio (it's a MASK)
// 0x0f is the max allowed, here...
#define MAX_MASK   0x0f

#define WRITE_TRACKS 80

#define DOAUDIO

//THIS MAY NOT BE PROPERLY-IMPLEMENTED
//#define SLOWTIMER

#ifdef SLOWTIMER
 #define TIMERDIV 8
 #define TCCR0B_CS   (1<<(CS01)) // DIV8
 #define TCCR1B_CS   ((1<<(CS12)) | (1<<(CS10))) //DIV16 (32MHz?!)
#else
 #define TIMERDIV 1
 #define TCCR0B_CS   (1<<(CS00)) // DIV1
 #define TCCR1B_CS   (1<<(CS11)) // DIV2 (For some reason it's 32MHz?)
#endif


// The idea is 5us per PWM-nibble (200kNibbles/sec)
//   where the first and last 2us are buffers
//   and the middle 1us is used for data

//   |---2us----------|--1us--|---   >2us  -----|
//    ________________ _ _ _ _
//   /                \_\_\_\_\________________/
//
//   then /WD and /RD will show the following:
//  _   ______________ _ _ _ _ _ ______________
//   \_/              \_X_X_X_X_/              \_/
//
//  e.g. '0x0':
//  _   ______________   ______________________
//   \_/              \_/                      \_/
//  and '0xf':
//  _   ______________________   ______________
//   \_/                      \_/              \_/
//   |

// Simplest method:
// OCR1C determines PWM-count
// OCR1B determines PWM-duty-cycle (low period)
// Data 0x0:
//   OCR1B = (pulse-duration)
//   OCR1C = 2us
//   Complete cycle
//   OCR1C = 3us
// Data 0xf:
//   OCR1C = 3us
//   Complete Cycle
//   OCR1C = 2us
//
// Then, generally:
//   OCR1C[0] = 2us + (dataVal)*(1us/16)
//   OCR1C[1] = 5us - OCR1C[0]
//   (OR)
//   OCR1C[1] = 3us
//
//at 16MHz, that's 16 clock-cycles per us...
// except, it seems that timer1 is running at 32MHz
// (Duh, just bump its prescaler to /2!)

// NOTE!!!!
// The duration of OCR1C[1] doesn't need to be precise!
// The only thing that matters, as far as storing/reading data,
// is the duration between the Start of the PWM cycle, and the "Duty-Cycle"
// edge... The next nibble could be *much* later.
// And Now, math is easier for the uC, loop-tests can be performed, etc.
// HOWEVER: it is (now) even more necessary to have synchronization to
// detect the start-pulse...

//#define WD_PULSE_DURATION 2 // ~> 0.1us (recalled from spec)
//#define T1_us 16
#warning "HER"
//?!?!?! Changing this from 0x70 to 48 made a *HUGE* difference
// Could it be related to the 8us Max requirement for WG and WD?
// version 55, now... been working on this since the early 40's...
// 48 is 3*16 = 3us...
// attempting 8us, now garbo again
#define PWM_DATA_STOP_LENGTH  (3*16) //48//0x70
//0xf0  // loaded into OCR1C, 2us is too small
                                    // for our system
                                    // For read-back, 0xff is a risky-value
                                    // too close to its limit with ICPvals

#if 1


#define wait(n) \
{ \
   uint32_t i; \
   for(i=0; i<n; i++) \
   { \
      asm("nop"); \
   } \
}

#if 1
//!!! This is no-good!
//    It only functions BEFORE timer1 is used for generating PWM!
void t1_delay_ms(uint32_t ms)
{
   
   uint32_t overflowCount=0;

   uint32_t totalCount = ms * (F_CPU/256/TIMERDIV) / 1000;
   //16MHz / 256 = 62500 = 1sec
   for(overflowCount=0; overflowCount < totalCount; overflowCount++)
   {

      TIFR = (1<<TOV1);
      while( !(TIFR & (1<<TOV1)) )
      {
         asm("nop");
      }
   }

}
#endif //0

// For Read-Back...
//
//   |---2us----------|--1us--|---   >2us  -----|
//    ________________ _ _ _ _
//   /                \_\_\_\_\________________/
//
//   then /WD and /RD will show the following:
//  _   ______________ _ _ _ _ _ ______________
//   \_/              \_X_X_X_X_/              \_/
//
//  e.g. '0x0':
//  _   ______________   ______________________
//   \_/              \_/                      \_/
//  and '0xf':
//  _   ______________________   ______________
//   \_/                      \_/              \_/
//   |
//
//  Reset Timer0 upon the first falling-edge
//  Use Input-Capture to determine the time until the next falling-edge
//
//  HOW to determine which edge is which?
//
//  A duration *longer* than 3us (say 4us) can be used to repeatedly
//   indicate a new start...
//  (TODO: Add this note to WRITE)
//  RESETTING Timer0 upon an edge isn't in the cards...
//  BUT: Could have it free-running, then math two separate ICP events


void t0_delay_ms(uint32_t ms)
{
   
   uint32_t overflowCount=0;

   uint32_t totalCount = ms * (F_CPU/256/TIMERDIV) / 1000;
   //16MHz / 256 = 62500 = 1sec
   for(overflowCount=0; overflowCount < totalCount; overflowCount++)
   {

      TIFR = (1<<TOV0);
      while( !(TIFR & (1<<TOV0)) )
      {
         asm("nop");
      }
   }

}

#if 0
uint8_t writeDataNibble = 0;

static __inline__ void nWR_t1_overflow_Handler(void)
   __attribute__((__always_inline__));


static __inline__ void nWR_t1_overflow_Handler(void)
{
   static uint8_t state = 0;

   switch(state)
   {
      //First pulse, start of PWM cycle -> PWM-duty
      case 0:
         OCR1C = 2*T1_us + writeDataNibble;
         state=1;
         break;
      //Second pulse, PWM-duty -> start of next PWM cycle
      case 1:
      default:
         OCR1C = 5*T1_us - OCR1C;
         state=0;
         break;
   }
}
#endif //0

#endif


//According to the TEAC 235 manual:
//STEP's rising-edge -> nWG's falling-edge
#define nSTEP_TO_nWG_MIN_DELAY_MS    18
// Apparently a flux-change MUST happen within 8us from /WG
// /WG falling-edge -> /WD rising-edge (!?)
#define nWG_TO_nWD_MAX_DELAY_US     8
// /WD falling-edge -> /WG rising-edge
#define nWD_TO_nWG_MAX_DELAY_US     8
// /STEP rising-edge -> /RD valid
#define nSTEP_TO_nRD_MAX_DELAY_MS    18
// /WG rising edge -> /STEP rising-edge
#define nWG_TO_nSTEP_MIN_DELAY_US   650
// /STEP rising-edge -> /STEP rising-edge
#define nSTEP_TO_nSTEP_MIN_DELAY_MS 3

// Number of times to prepend a data-sequence with "SYNC"
#define NUM_SYNCS    0xf7

void startTimer0(void)
{
   TCCR0A = (1<<ICEN0); //Enable Input-Capture, Falling-edge, 8-bit

   TCCR0B = TCCR0B_CS | (1<<ICNC0);
      //(1<<CS00);  //No prescaling... runs at FCPU (right? YES)
      //Noise Cancelling
}

void pwmifyHeart(void)
{
   //We're showing a nibble... so make it easy!
   OCR1C = 0x0f;
   //Set on compare match
   //Clear at 0x000
   TCCR1A |= (1<<COM1A1) | (1<<COM1A0) ;
   
   TCNT1 = 0;

   //Enable PWM on OC1A
   TCCR1A |= (1<<PWM1A);
}


#define audioSetPWM(nibble) \
{ \
   OCR1B = (nibble & 0xf) ; \
}

#define heartSetPWM(nibble) \
{ \
  OCR1A = (nibble & 0xf) ; \
}


#if 0
void readDataToLED(void)
{
   pwmifyHeart();

   //gather two ICP values
   //nibble (or 0xf-nibble) == ICP1 - ICP0 - 2us
   //Write that to the heartPWM...

   while(1)
   {
      //Grab the first ICP-value
      while(!(TIFR & (1<<ICF0)))
      {
         asm("nop");
      }
      TIFR = (1<<ICF0);
      uint8_t firstVal = OCR0A; //ICR0;
      //Then the second...
      while(!(TIFR & (1<<ICF0)))
      {
         asm("nop");
      }
      TIFR = (1<<ICF0);
      uint8_t secondVal = OCR0A; //ICR0;
      
      //Technically, this could either be nibbleVal or 0xf-nibbleVal...
      // since syncing is NYI
      // 2us (buffer) is 32 cycles.
      uint8_t nibblePrime = secondVal - firstVal - 32;

      heartSetPWM(nibblePrime);

   }

}
#endif //0



 #define WD_PULSE_DURATION 10 //2     // 2/16 > 0.1us...
 #define T1_us  16

void startTimer1(void)
{
   OCR1B = WD_PULSE_DURATION;

   //OC1WB Set on Compare Match
   //      Clear when TCNT1 = 0x000
   //      OC1B connected, /OC1B disconnected
   // (Table 12-14 FAST-PWM)
   TCCR1A |= (1<<COM1B1) | (1<<COM1B0) ; 

   //Set the timer to overflow at 0xff (8-bit)
   //BEWARE THIS WILL BE VARIED DURING nWD!!! (and heart PWM)
   OCR1C = 0xff;

   //Make sure TCNT1 starts at 0...
   TCNT1 = 0;
   //PLLCSR = (1<<PCKE) | (1<<PLLE) ;

   //Clock Select: CS13:10 = 0 0 0 1 = PCK / CK (depending on PLL...?)
//#warning "TEST PCK vs CK!!!"
   // WANT: 16MHz
   //This selects the prescaler (and starts counting?)
#if 0
   //This *should* be CK = 16MHz, no...? 'cause it's acting like CK*2
   TCCR1B |= (1<<CS10);
#else 
#if 1
   //For some reason it's acting like CK*2=32MHz
   // So, divide by 2, to sync with the system-clock
   // (will make calculations quicker)
   TCCR1B |= TCCR1B_CS; //(1<<CS11);
#else
   //For testing, we should get 16MHz / 16384 or ~977 counts / sec
   // or 3.8 (0xff) overflows / sec
   // BUT, it's going twice as fast as expected. WTF...
   TCCR1B |= (1<<CS13) | (1<<CS12) | (1<<CS11) | (1<<CS10) ;
#endif
#endif

   //This contains only shadow-bits and OCR1D configuration
   //TCCR1C;

   //WGM configures FAST-PWM mode, but 00 is boot-default...
   //TCCR1D |= ;

   //PLL Control/Status
   // LSM can't be set if PLL is sysclk
   // PCKE...
   // This should default to timerclk = sysclk... no?
   //PLLCSR = 

   
   // Enable PWM mode based on OCR1B
   TCCR1A |= (1<<PWM1B) ;
}

//Away from the center, toward Track 0 (sensor at outter edge of diskette)
//These have been verified with TEAC manual...
#define stepOut() setpinPORT(nDIR_PIN, nDIR_PORT)
//Away from Track 0
#define stepIn()  clrpinPORT(nDIR_PIN, nDIR_PORT)

void doStep(void)
{
   clrpinPORT(nSTEP_PIN, nSTEP_PORT);
   t0_delay_ms(1);
   setpinPORT(nSTEP_PIN, nSTEP_PORT);
}


int main(void)
{
   cli();
   //ATTINY861:
   // using custom fuses in makefile should use PLL at 16MHz...
   // doesn't seem to be...
   // "The internal PLL is enabled when the CKSEL fuse bits are 0001
   //  and the PLLE bit of PLLCSR is set"
   // "If PLL is selected as a system clock source the value for this bit
   //  is always 1"
   // ... so, nevermind, then...
   // test of delay_ms shows near-exact 16MHz
   //  (The problem musta been the shitty heartbeat example using dms)

   init_io();

   startTimer1();
   startTimer0();


   //### STEP In
   clrpinPORT(HEART_PINNUM, HEART_PINPORT);
   uint8_t stepCount = 0;

   stepIn();

   for(stepCount=0; stepCount<100; stepCount++)
   {
      t0_delay_ms(100);
      doStep();
   }

   stepCount=0;


   //### WAIT 2 SECONDS (TIMER 0)
   setpinPORT(HEART_PINNUM, HEART_PINPORT);
   t0_delay_ms(2000);
   
   //### STEP BACK Out UNTIL TRK0 is detected
   clrpinPORT(HEART_PINNUM, HEART_PINPORT);
   stepOut();

   while(getpinPORT(nTRK0_PIN, nTRK0_PORT))
   {
      t0_delay_ms(50);
      doStep();
   }

   //WAIT AGAIN (TIMER 1)
   setpinPORT(HEART_PINNUM, HEART_PINPORT);
   //ONLY use t1_delay_ms() HERE (nowhere else)
   // just to test that it's running at the speed we expect
   t1_delay_ms(2000);

   stepIn();

#define DOWRITE

#ifdef DOWRITE
#define PWM_DATA_SYNC_US   5 //10 may've given more static in beginning
#define PWM_DATA_BUFFER_US 3 //4 //2
#define PWM_DATA_WIDTH_US  1
   for(stepCount=0; stepCount<WRITE_TRACKS; stepCount++)
   {
      uint8_t dataNibble = stepCount & 0x0f;

      togglepinPORT(HEART_PINNUM, HEART_PINPORT);
      
      //Wait for the index pulse
      while(getpinPORT(nIDX_PIN, nIDX_PORT))
      {
         asm("nop");
      }
      //Then wait for it to clear
      while(!getpinPORT(nIDX_PIN, nIDX_PORT))
      {
         asm("nop");
      }

      //Make sure the PWM /OUTPUT/ is HIGH
      //*BEFORE* Enabling the write-gate
       //STOP PWM MODE AND COUNTING
        //Stop PWM-Mode, now we're in normal/WGM mode (FOC1 will work)
        TCCR1A &= ~(1<<PWM1B);
        //Give us plenty of time...
        OCR1C = 0xff;
        //Clear the overflow flag
        TIFR |= (1<<TOV1);
        //Wait for an overflow, so we know where we're at...
        while ( !(TIFR & (1<<TOV1)) )
        {
           asm("nop");
        }
        //Now we'll have a low-value output...
        //Force an output-compare so the pin is high
        // (All the above, 'cause we dunno whether FOC1 works when stopped)
        TCCR1A |= (1<<FOC1B);

        //WGM is don't-care (normal non-pwm mode)
        //COM1B1:0 = 11 Set on compare-match, clear when TCNT1=0 (FAST-PWM)
        // In NORMAL mode, this corresponds to Set On Compare-Match
        //The timer is still running (in normal-mode)
        //Stop the timer
        TCCR1B = 0;
        //Make sure it's not double-buffered or something.
        while (TCCR1B != 0)
        {
           asm("nop");
        }
        //Clear the Overflow Flag while the timer is nowhere near and
        //overflow and stopped...
        TIFR |= (1<<TOV1);

       //Make sure when it's restarted, we don't get a compare-match
       //and have plenty of time to setup the first nibble
       OCR1C = 0xff;
       TCNT1 = OCR1B+1;

       //The pin is now high, the timer is stopped
       // The timer will start *after* output-compare, so will remain high
       // until the next overflow (0xff) (we'll have about 100
       // instruction-cycles)

       // Set PWM-MODE:
       TCCR1A |= (1<<PWM1B);

       // START THE TIMER (/2 for some reason)
       TCCR1B = TCCR1B_CS; //(1<<CS11); 

       //We have about 200 cycles until the PWM-Nibble starts
       // FROM THIS POINT

       //DOUBLE-BUFFERING IS NOW ON
       //SET UP THE FIRST PWM-DATA-SYNC-CYCLE (Start -> Sync)
       OCR1C = PWM_DATA_SYNC_US * T1_us; //2*T1_us + dataNibble;
       //This won't start until AFTER the current PWM-cycle which has been
       //de-pwm-pulse-ified via FOC1B, etc. above, and will be really long

       // /WG->/WD < 8us = 128 clock cycles
      
       //The first /WD pulse should be received less than 8us after /WG
       // We should have *plenty* of time until that first pulse
       // So *WAIT* until there will be less than 8us!
       while ( TCNT1 < 0xff - nWG_TO_nWD_MAX_DELAY_US * T1_us)
       {
          asm("nop");
       }

      //Enable WRITING
      clrpinPORT(nWG_PIN, nWG_PORT);


      //NOW: /WG is enabled
      //     The first PWM-Data-SYNC pulse will arrive in less than 8us
      // We'd like to allow the SYNC pulse to repeat a few times.

      uint8_t syncCount;
      for(syncCount=0; syncCount<NUM_SYNCS-1; syncCount++)
      {
         //Wait until the PWM-Data-Sync cycle begins
         while( ! (TIFR & (1<<TOV1)) )
         {
            asm("nop");
         }
         //Clear the flag
         TIFR |= (1<<TOV1);
      }

      //Measuring syncCount requires watching TOV1
      // which requires a lot of cycles
      // if the Sync width is, say, 5us, that's 80 CPU cycles
      // which should be plenty for a simple loop
      // but might not leave enough overhead for exitting the loop and
      // beginning the actual data
      //SO: Repeat the TIFR-watch outside the loop
      //    Allowing for yet another sync
      //    And setup the DATA afterwards
      //Wait until the PWM-Data-Sync cycle begins
      while( ! (TIFR & (1<<TOV1)) )
      {
         asm("nop");
      }
      //Clear the flag
      TIFR |= (1<<TOV1);
      
      //NOW:
      // We've JUST(ish) received indication that a new PWM-SYNC cycle has
      // begun... (The pin is low and will return high soon, remaining so
      // for a total of 5us)
      // So, we've got LESS THAN 5us to load in the data-value
      // that will start the data-sequence after TOV1
      // (but can do-so somewhat freely, due to double-buffering)
      // (As long as it's BEFORE that 5us is over)
      // SOME of that 5us was used in testing and clearing TOV1!
      OCR1C = PWM_DATA_BUFFER_US*T1_us + dataNibble;
      
      //It Gets Better!
      // Because ONCE data is begun, we'll have only 2-3us between
      // PWM-loads!


      //Wait until the PWM-Data-Start cycle begins
      while( ! (TIFR & (1<<TOV1)) )
      {
         asm("nop");
      }
      //Clear the flag
      TIFR |= (1<<TOV1);


      //The PWM-Data cycle has begun... 
      // the pin has gone low to indicate start... 
      // it will go high in about 0.1us
      // then remain high, until the PWM-Data cycle completes
      // AT WHICH POINT:
      // A new uC-PWM cycle will begin, pulling the pin low briefly
      //  (indicating the data-value)
      // then remaining high until the cycle completes
      // as determined by OCR1C *at that time*
      // BUT: Since OCR1C is double-buffered
      // We can/should set the new OCR1C value NOW.
      //
      // If we set a LARGE value for OCR1C
      // Then we can *interrupt* the high-portion of the cycle
      // when we're ready to start a new PWM-Data cycle.
      // (e.g. after while() testing, etc.)
      // This'll give about 200 CPU cycles to figure all that out.
      // (Hopefully far less, for the sake of storage capacity)
      // This *after* the PWM-Data cycle *completes*...
      // So, *plenty* of time, no?
      OCR1C = PWM_DATA_STOP_LENGTH;

#warning "TODO: FOR NOW we're not worried about capacity, so we're not going to worry about how to optimize or interrupt the PWM-DATA-STOP cycle"



      //NOW:
      // The first PWM-Data-Nibble has been STARTED
      // Its data-cycle is near completion
      // AFTER which, it will begin the STOP sequence

      // Let us begin a loop for remaining sequential data
      // The procedure is identical, but I want to leave the comments above
      // for the syncing procedure, specifically.

//#######
      //200KHz
      uint8_t count=0;
      do
      {

         //Wait until the last PWM-Data-STOP cycle begins
         while( ! (TIFR & (1<<TOV1)) )
         {
//            asm("nop");
         }
         //Clear the flag
         TIFR |= (1<<TOV1);
      


         //NOW:
         // The PWM-Data STOP cycle has begun
         // It's time to pre-load the next data-value

         // We've got PLENTY of time because we're not yet worried about
         // storage capacity... The STOP cycle is quite long
         //  and, as-yet uninterruptable.

#ifdef DOAUDIO
         count+=stepCount;
         if(count > (255-80)-stepCount)
         {
            static uint8_t moreDelay=0;
            moreDelay++;
            if(moreDelay > 10)
            {
               dataNibble++;
               dataNibble &= MAX_MASK;
               moreDelay = 0;
            }
            count=0;
         }
#endif
         OCR1C = PWM_DATA_BUFFER_US*T1_us + dataNibble;
      
         //It Gets Better!
         // Because ONCE data is begun, we'll have only 2-3us between
         // PWM-loads!


         //Wait until the PWM-Data-Start cycle begins
         while( ! (TIFR & (1<<TOV1)) )
         {
//            asm("nop");
         }
         //Clear the flag
         TIFR |= (1<<TOV1);


         //The PWM-Data cycle has begun... 
         // We can set the new OCR1C value for PWM-Data-STOP NOW.
         
         // This'll give about 200 CPU cycles to figure all that out.
         OCR1C = PWM_DATA_STOP_LENGTH;
      
      } while(getpinPORT(nIDX_PIN, nIDX_PORT));
//#######

      //Now, there's a PWM-Data-STOP cycle in waiting
      //Wait until that begins
      while( ! (TIFR & (1<<TOV1)) )
      {
//         asm("nop");
      }
      //Clear the flag
      TIFR |= (1<<TOV1);

      //The /WG output should be disabled WITHIN 8us from the last
      // low-pulse on /WD...
      // WHICH is *just now* occurring (or maybe just past)
      //WEIRD: The TEAC documentation shows this to be a MAXIMUM of 8us
      // FROM the FALLING-edge of /WD
      // This occurs when TOV1
      // So's I guess we can just *do it* now...
      // The above shouldn't take more than 8us, otherwise we got probles
      // elsewhere.

      //DISABLE WRITING
      setpinPORT(nWG_PIN, nWG_PORT);

      //Note that the PWM pin, on /WD, will continue repeating the last
      //value... Oh Well.
      t0_delay_ms(1); //nWG_TO_nSTEP_MIN_DELAY_US);
      doStep();
      t0_delay_ms(nSTEP_TO_nWG_MIN_DELAY_MS);
   }

   //Stop the PWM output from changing
   //TCCR1A &= ~( (1<<PWM1B) | (1<<COM1B1) | (1<<COM1B0) );
   //BUT PWM is still running on the LED...
   //TCCR1B = 0;
   

#endif


#define DOREAD
#ifdef DOREAD
   stepOut();


   //Fade the LED showing that PWM's working
   pwmifyHeart();
   uint8_t pwmVal;
   for(pwmVal=0; pwmVal<0x10; pwmVal++)
   {
      heartSetPWM(pwmVal);
      t0_delay_ms(250);
   }

   heartSetPWM(0);
   t0_delay_ms(2000);

   //Beep the speaker, showing the speaker's working
   uint16_t timer;
   for(timer=0; timer<200; timer++)
   {
      audioSetPWM(0);
      t0_delay_ms(1);
      audioSetPWM(8);
      t0_delay_ms(1);
   }

   heartSetPWM(0xf);
   t0_delay_ms(2000);

   //Simulate what should be written to disk...
{
   uint8_t stepCountFAKE;
   for(stepCountFAKE=0; stepCountFAKE<80; stepCountFAKE++)
   {
      uint8_t dataNibbleFAKE = 0;

      uint8_t count=0;
      //do
      uint32_t time;
      for(time=0; time<65536; time++)
      {
         count+=stepCountFAKE;

         if(count > (255-80)-stepCountFAKE) //255-2*stepCount)
         {
            dataNibbleFAKE++;
            dataNibbleFAKE &= 0x0f;
            count=0;
            audioSetPWM(dataNibbleFAKE);
         }
      }
   }
}

   heartSetPWM(0x0f);
   t0_delay_ms(2000);



   //Begin readback
   OCR1A = 0x0f;

   stepCount = 0;
   //This should be a do-while... but, it's kinda nice that the last track
   //isn't written (eh?!) No, this is READ... huh...
   // So, write, then, might be nice like this, and might otherwise should
   // be a do-while... or something.
   //HERE: we don't get to hear Track0...
   do //while(getpinPORT(nTRK0_PIN, nTRK0_PORT))
   {

      //Set a default PWM...
      // Then we can see it changing each step
      // ...if it's not getting any data
      //This is kinda hokey a/o audioSetPWM...
      // but it won't hurt...
      heartSetPWM(stepCount);
      //OCR1A = (stepCount & 0x0f);
//      uint32_t i;
      //Roughly 2.5seconds per track
      // Each PWM-nibble is 5us long... that's 200,000 per second
      // 1,000,000 should be 5 seconds... how'd I come up with 2.5?
      // Something was doubled... I think I was originally planning to
      // handle each ICP individually per loop.
      // BUT, that doesn't account for its taking roughly 10 seconds
      //  Plausibly the for-test is making it miss some...
      //  which could also account for flickering.
//      for(i=0; i<500000; i++)
//#error "NOT IN THE FOR LOOP"
      uint8_t idxCount;
      for(idxCount=0; idxCount < 5; idxCount++)
      {
#if 0
         //Wait for an edge
         while(getpinPORT(nRD_PIN, nRD_PORT))
         {
            asm("nop");
         }
         //And for it to clear
         while(!getpinPORT(nRD_PIN, nRD_PORT))
         {
            asm("nop");
         }
         togglepinPORT(HEART_PINNUM, HEART_PINPORT);
#else

         //We've just begun reading...
         // but aren't necessarily at the beginning of the track
         // So wait for index to pulse...

         while(getpinPORT(nIDX_PIN, nIDX_PORT))
         {
            asm("nop");
         }
         //I dun recall why, but writing isn't started until after it's
         //cleared...
         //Prb to assure it doesn't start writing randomly in the middle
         while(!getpinPORT(nIDX_PIN, nIDX_PORT))
         {
            asm("nop");
         }
         //A bunch of setup occurs during writing...
         // So let's just add a bit of a delay..
         // 5us for SYNC pulses, and 0xff pulses is ~1.275ms
         // but t0_delay_ms is probably not precise enough...
         //t0_delay_ms(1);
         //_delay_us(500); //WTF maximal delay = "768us / FCPU" ???
         uint16_t i;
         //Just guessing... say there's 100 instructions before it starts
         // and 200 delay from initializing the PWM
         // and trying to keep well-under the 1ms...
         for(i=0; i<300; i++)
         {
            asm("nop");
         }

         //Now, hopefully, this lands us in the middle of the SYNC stuff
         


         //Clear the Input-Capture flag
         TIFR = (1<<ICF0);

         
         
         //Wait for the first input-capture
         while(!(TIFR & (1<<ICF0)))
         {
//           asm("nop");
         }
         //Clear the Input-Capture flag (quickly)
         TIFR = (1<<ICF0);

         //Initialize lastVal for the first go-round...
         uint8_t lastVal = OCR0A;

         do //until index...
         {
            //Wait for the next input-capture
            while(!(TIFR & (1<<ICF0)))
            {
 //              asm("nop");
            }
            //Clear the Input-Capture flag (quickly)
            TIFR = (1<<ICF0);

            //get this value...
            uint8_t thisVal = OCR0A;

            //Process the difference
            
            /*
            // (Note that T0 and T1 are both running at FCPU)
            uint8_t nibblePrime = thisVal - lastVal 
                                  - PWM_DATA_BUFFER_US * T1_us;
            lastVal = thisVal;

            //Then we have data...
            if(nibblePrime <= PWM_DATA_WIDTH_US * T1_us )
            {
               heartSetPWM(nibblePrime);
            }
            */
            //BECAUSE PWM_DATA_WIDTH_US is a multiple of 16 (T1_us)
            // AND BECAUSE our nibble is limited to/masked with 0x0f
            // We don't NEED to subtract PWM_DATA_BUFFER_US!!!
            // This'll save a lot of overhead!
            uint8_t nibblePrime = thisVal - lastVal; 
                                 //- PWM_DATA_BUFFER_US * T1_us;
            lastVal = thisVal;
            if( ( nibblePrime < 
                  (PWM_DATA_WIDTH_US + PWM_DATA_BUFFER_US) * T1_us )
//#ifdef SLOWTIMER
                  && (nibblePrime >= PWM_DATA_BUFFER_US*T1_us)
//#endif 
               )
            {
               audioSetPWM(nibblePrime); // & 0x0f);
            }
         } while (getpinPORT(nIDX_PIN, nIDX_PORT));

#endif

      }
      
      t0_delay_ms(50);
      doStep();
      t0_delay_ms(50);
      stepCount++;
   } while(getpinPORT(nTRK0_PIN, nTRK0_PORT));

   OCR1A = 0x00;
   t0_delay_ms(1000);
   OCR1A = 0x0f;
#endif



   //### BACK AND FORTH, TOGGLE LED WHEN TRK0 DETECTED
   stepCount=0;
   while(1)
   {
      if(!getpinPORT(nTRK0_PIN, nTRK0_PORT))
      {
         if(OCR1A)
            OCR1A = 0;
         else
            OCR1A = 0x0f;
         //togglepinPORT(HEART_PINNUM, HEART_PINPORT);
      }

#if 0
      //After a step, don't allow another step until we read something
      // (Just to make sure the /RD pin isn't blocked during settling)
      while(getpinPORT(nRD_PIN, nRD_PORT))
      {
         asm("nop");
      }
#endif

      //AND Also wait for the index...
      while(!getpinPORT(nIDX_PIN, nIDX_PORT))
      {
         asm("nop");
      }
      while(getpinPORT(nIDX_PIN, nIDX_PORT))
      {
         asm("nop");
      }

      doStep();

      t0_delay_ms(50);
      
      stepCount++;
      
      if(stepCount == 10)
      {
         togglepinPORT(nDIR_PIN, nDIR_PORT);
         stepCount=0;
      }

   }



   return 0;
}


void init_io(void)
{

setoutPORT(HEART_PINNUM, HEART_PINPORT);

//OC1B nWD output Shared with Speaker output
setoutPORT( nWD_PIN, nWD_PORT);
//setoutPORT( SPK_PIN, SPK_PORT);

//OC1D output
setoutPORT( nWG_PIN, nWG_PORT);

//INT0 input
setinpuPORT( nIDX_PIN, nIDX_PORT);

//ADC0 input (analog)
setinPORT( LINE_IN_PIN, LINE_IN_PORT);

//GPIO input
setinpuPORT( nTRK0_PIN, nTRK0_PORT);

//ICP0 input
setinpuPORT( nRD_PIN, nRD_PORT);

//GPIO outputs
setoutPORT( nDIR_PIN, nDIR_PORT);
setoutPORT( nSTEP_PIN, nSTEP_PORT);
setoutPORT( nHEAD_PIN, nHEAD_PORT);
   



//OC1B nWD output Shared with Speaker output
setpinPORT( nWD_PIN, nWD_PORT);
//setoutPORT( SPK_PIN, SPK_PORT);

//OC1D output
setpinPORT( nWG_PIN, nWG_PORT);

//INT0 input
setpuPORT( nIDX_PIN, nIDX_PORT);

//ADC0 input (analog)
//setpuPORT( LINE_IN_PIN, LINE_IN_PORT);

//GPIO input
setpuPORT( nTRK0_PIN, nTRK0_PORT);

//ICP0 input
setpuPORT( nRD_PIN, nRD_PORT);

//GPIO outputs
setpinPORT( nDIR_PIN, nDIR_PORT);
setpinPORT( nSTEP_PIN, nSTEP_PORT);
setpinPORT( nHEAD_PIN, nHEAD_PORT);

}
#warning "WTF... timer1 seems to be counting at 32MHz, whereas the system   clock is supposedly 16MHz, despite disabling PCKE. Sysclk=16MHz somewhat   confirmed by use of _delay_ms() being relatively accurate with FCPU"
#if 0
      //Begin PWM-Nibble Sequence NOOOOOO
      //OCR1C is *double-buffered* (yes? A, B, and D are "x"... so prb)
      //So its value will be valid *after* TCNT1 becomes 0
      //### ACTUALLY, since this is the *START* of an entire
      //    sequence... we might as well not begin with data
      //    A PWM-Nibble "start" can be determined by its distance from the
      //    last pulse... >3us
      OCR1C = 4*T1_us; //2*T1_us + dataNibble;
      TCNT1 = 0;
      
      //TODO: Here's where we Do a few syncing pulses first! revise above
      //literature


      // ALLOW THE 4us sync pulse to repeat several times...
      uint8_t syncCount;
      for(syncCount=1; syncCount<0xff; syncCount++)
      {
         //Clear TIFR so we can watch for it...
         // (This'll occur at the sync-edge, at the end of the first
         //  PWM
      // cycle setup by OCR1C above)
      TIFR = (1<<TOV1);


      }

#endif //0
#if 0

      do
      {
         uint8_t tifrVal = (1<<TOV1);
         uint8_t ocr1cVal0 = 2*T1_us + dataNibble;


         //The duration between the data-edge and the next start-edge
         // doesn't need to be specific
         // Math would be easier on the uC if a constant value was set
         // (e.g. 3us)
         // HOWEVER: It occurs to me, now, that the loop-tests, etc. may
         // take an unpredict[ed] amount of time...
         // SO: Could it be easier to set ocr1cVal1 to *way too long*
         // Then interrupt that cycle when ready...?
         //uint8_t ocr1cVal1 = 5*T1_us - ocr1cVal0;


         //Begin Loop (after tests)
         // reset timer (TCNT1 = 0, prescaler reset?)
         //   (TCCR1B |= (1<<PSR1) ?)
         // Clear at TCNT1==0
         // Set after 0.1us (constant OCR1B)
         // Remain set until next edge (data-val)
         //   (determined by ocr1cVal0)
         // AT OVERFLOW (TCNT1 == ocr1cVal0)
         //   Load new ocr1cVal1
         // Output clears (TCNT1 == 0)
         // Sets after 0.1us
         // Remain set until next edge (start of new nibble)
         //   (ocr1cVal1 could be quite large)
         // WHEN DATA's READY:
         //  (AND TESTS PERFORMED)
         //  Loop.

         // OCR1x == DOUBLE-BUFFERED (when PWM)
         //   Updates at top/bottom of count-sequence
         //
         // Force Output Compare ("Non-PWM")
         //  Does NOT set OCF1x flag, nor reload/clear TCNT1
         //
         // TCNT1 write: Blocks Compare-Matches in next cycle

         uint8_t ocr1cVal1 = 0xff;

         __asm__ __volatile__
         (
            "sts %2, %0; \n\t"
            "sts %3, %1; \n\t"
            :
            : "r" (tifrVal),  //"%0"
              "r" (ocr1cVal0), //"%1"
              "M" (_SFR_MEM_ADDR(TIFR)),  //"%2"
              "M" (_SFR_MEM_ADDR(OCR1C)) //"%3"
         );



         //TIFR = (1<<TOV1);
         //nWR_t1_overflow_Handler();
         //OCR1C = 2*T1_us + dataNibble;

         //Wait until the next overflow
         while( ! (TIFR & (1<<TOV1)) )
         {
         //   asm("nop");
         }

         __asm__ __volatile__
         (
            "sts %2, %0; \n\t"
            "sts %3, %1; \n\t"
            :
            : "r" (tifrVal),  //"%0"
              "r" (ocr1cVal1), //"%1"
              "M" (_SFR_MEM_ADDR(TIFR)),  //"%2"
              "M" (_SFR_MEM_ADDR(OCR1C)) //"%3"
         );

         //TIFR = (1<<TOV1);
         //nWR_t1_overflow_Handler();
         //OCR1C = 5*T1_us - OCR1C;
         
         //Wait until the next overflow
         while(!(TIFR & (1<<TOV1)))
         {
            //asm("nop");
         }
      } while(getpinPORT(nIDX_PIN, nIDX_PORT));

/*
      //And finish the track...
      while(!getpinPORT(nIDX_PIN, nIDX_PORT))
      {
         //Wait until the next overflow
         while(!(TIFR & (1<<TOV1)))
         {
            asm("nop");
         }
         TIFR = (1<<TOV1);
         nWR_t1_overflow_Handler();
      }
*/

#endif //0
